#!/bin/python
# -*- coding: utf-8 -*-
###############################################################################
# duinobot-interactive
# Conjunto de clases python para permitir que el robot Multiplo Duinobot N6
# pueda programarse en forma interactiva, sin necesidad de bajar firmware al
# robot. Sin embargo es necesario el soporte del firmware del lado del robot
#
# Copyright (C) 2012 Fernando López <flopez AT linti.unlp.edu.ar>
# Copyright (C) 2011 David Vilaseca <http://www.robotgroup.com.ar>
# Copyright (C) 2011 Joaquín Bogado <jbogado en linti.unlp.edu.ar>
# 
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program. 
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################

from pyfirmata import DuinoBot, util
import time,re, os
import threading

class Board(object):
    lock = threading.Lock()
    def __init__(self, device='/dev/ttyUSB0'):
        '''Inicializa el dispositivo de conexion con el/los robot/s'''
        self.board = DuinoBot(device)
        it = util.Iterator(self.board)
        it.start()
        self.board.pass_time(0.1)

    def __del__(self):
        self.exit()
    
    def __getattribute__(self, name):
        '''Permitir multithreading para evitar problemas con el método
        senses'''
        Board.lock.acquire()
        try:
            res = object.__getattribute__(self, name)
        except:
            Board.lock.release()
            raise
        Board.lock.release()
        return res

    def motor0(self,vel,robotid):
        if(vel >= 0 and vel <= 100):
            self.board.send_sysex(1,[int(vel), robotid])    

    def motor1(self,vel,robotid):
        if(vel >= 0 and vel <= 100):
            self.board.send_sysex(2,[int(vel), robotid])   
            
    def motors(self,vel1, vel2, seconds=-1, robotid=0):
        if(abs(vel1)<=100 and abs(vel2)<=100):
            self.board.send_sysex(4,[int(abs(vel1)), int(abs(vel2)), 1 if vel1>0 else 0, 1 if vel2>0 else 0, robotid])   
            if seconds!=-1:
                self.board.pass_time(seconds)
                self.motors(0,0,-1,robotid)

    def ping(self,robotid):
        self.board.send_sysex(3,[robotid])
        # wait 20ms (ping delay) + 20ms (comm)
        self.board.pass_time(0.04)
        return self.board.nearest_obstacle[robotid]  

    def sleep(self,time):
        self.board.pass_time(time)

    def exit(self):
        self.board.exit()
        
    def analog(self,ch,samples=1,robotid=0): 
        self.board.send_sysex(6,[ch, samples, robotid])
        self.board.pass_time(0.04)
        return self.board.analog_value[robotid]

    def battery(self,robotid): 
        self.board.send_sysex(6,[6, 1, robotid])
        self.board.pass_time(0.04)
        return self.board.analog_value[robotid]*5.0/1024

    def digital(self,pin,robotid): 
        self.board.send_sysex(7,[pin,robotid])
        self.board.pass_time(0.04)
        return self.board.digital_value[robotid]
        
    def beep(self, freq=0, microseconds=0, robotid=0):
        hi=freq>>7
        lo=freq%128
        if freq!=0:
            if microseconds==0:
                self.board.send_sysex(5,[hi, lo, robotid])
            else:
                self.board.send_sysex(5,[hi, lo, int(microseconds*1000), robotid])
                #self.board.pass_time(microseconds)
        else:
            self.board.send_sysex(5,[robotid])

    def report(self): 
        self.board.live_robots=[0 for i in range(128)]
        self.board.send_sysex(9,[])
        self.board.pass_time(0.5)
        robotlist = []
        for i in range(128):
            if self.board.live_robots[i]:
                robotlist.append(i)
        return robotlist

    def set_id(self,new_id,robotid): 
        self.board.send_sysex(8,[new_id,robotid])
        self.board.pass_time(0.02)
    
    def getLine(self, robotid):
        a = self.analog(18,robotid=robotid)
        b = self.analog(19,robotid=robotid)
        return (a,b)

    def getWheels(self, robotid):
        a = self.analog(16,robotid=robotid)
        b = self.analog(17,robotid=robotid)
        return (a,b)
    
    wait = sleep

class Robot:
    def __init__(self, board, robotid=0):
        '''Inicializa el robot y lo asocia con la placa board.'''
        self.robotid = robotid
        self.board = board
        self.name = ''
        
    def __del__(self):
        self.board.exit()
        
    ##MOVIMIENTO
    def forward(self, vel=50, seconds=-1):
        '''El robot avanza con velocidad vel durante seconds segundos.'''
        self.board.motors(vel, vel, seconds, self.robotid)

    def backward(self, vel=50, seconds=-1):
        '''El robot retrocede con velocidad vel durante seconds segundos.'''
        self.forward(-vel, seconds)
    
    def motors(self, vel1, vel2, seconds=-1):
        '''Permite mover las ruedas independientemente, con velocidad vel1 para un motor y vel2 para el otro motor, durante second segundos.'''
        self.board.motors(vel1, vel2, seconds, self.robotid)

    def turnLeft(self, vel=50, seconds=-1):
       '''El robot gira a la izquierda con velocidad vel durante seconds segundos.'''
       self.board.motors(vel, -vel, seconds, self.robotid)

    def turnRight(self, vel=50, seconds=-1):
        '''El robot gira a la derecha con velocidad vel durante seconds segundos.'''
        self.board.motors(-vel, vel, seconds, self.robotid)

    def stop(self):
        '''Detiene todo movimiento del robot inmediatamente.'''
        self.board.motors(0, 0, robotid=self.robotid)

    ##SENSORES
    def getLine(self):
        '''Devuelve los valores de los sensores de linea.'''
        return self.board.getLine(self.robotid)

    def getWheels(self):
        '''Devuelve el valor de los sensores de las ruedas.'''
        return self.board.getWheels(self.robotid)

    def getWheelsB(self):
        (r,l) = self.getWheels()
        return (r/500,l/500)

    def getObstacle(self, distance=10):
        '''Devuelve True si hay un obstaculo a menos de distance
         centimetros del robot.'''
        ping = self.ping()
        if ping < 0:
            # Consideramos que una lectura fallida
            # del sensor retorna -1
            return False
        return ping < distance

    def battery(self):
        '''Devuelve el voltaje de las baterías del robot.'''
        return self.board.battery(self.robotid)

    def ping(self):
        '''Devuelve la distancia en centimetros al objeto frente al robot.'''
        return self.board.ping(self.robotid)

    def beep(self, freq=200, seconds=0):
        '''Hace que el robot emita un pitido con frecuencia freq durante seconds segundos.'''
        self.board.beep(freq, robotid=self.robotid)
        if seconds > 0:
            self.board.wait(seconds)
            self.board.beep(0, robotid=self.robotid)

    def sensors(self):
        '''Imprime informacion de los sensores.'''
        print 'Línea = ' + str(self.getLine()) 
        print 'Ruedas = ' + str(self.getWheels())
        print 'Obstaculo mas cercano = ' + str(self.ping()) + ' cm.'
        print 'Bateria = ' + str(self.battery()) + ' v.'
    
    ##Identificadores
    def setId(self, newid):
        '''Setea el robotid'''
        self.board.set_id(newid, self.robotid)
        self.robotid = newid
    
    def setName(self, name):
        '''Setea el nombre para el robot.'''
        self.name = str(name)
    
    def getId(self):
        '''Devuelve el robotid.'''
        return self.robotid

    def getName(self):
        '''Devuelve el nombre del robot.'''
        return self.name
    
    ## Otras funciones
    def speak(self, msj):
        '''Imprime en la terminal el mensaje msj.'''
        print msj

from senses import *
Robot.senses = senses

def wait(seconds):
    '''Produce un retardo de seconds segundos en los que el robot no hace nada.'''
    time.sleep(seconds)

__devPattern = re.compile(r"^ttyUSB\d+$")
def boards():
    matching = filter(__devPattern.match, os.listdir("/dev"))
    return map(lambda s: "/dev/" + s, matching)



# -*- encoding: utf-8 -*-

###############################################################################
# joystick.py
#
# Copyright (C) 2012 Fernando López <soportelihuen AT linti.unlp.edu.ar>
# 
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program. 
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################

import pygame
# Inicializar solamente el Joystick es más rápido pero suele fallar
pygame.init()

def joysticks():
	"Retorna una lista joysticks conectados"
	Joystick._Joystick__refresh()
	joys = []
	for id in range(pygame.joystick.get_count()):
		j = pygame.joystick.Joystick(id)
		joys.append((id, j.get_name(),))
		j.quit()
	return joys

class Joystick:
	"Permite controlar un robot con un joystick"
	joystickDetected = pygame.joystick.get_count() > 0
	@classmethod
	def __refresh(cls):
		"""Esta función permite conectar el joystick aun después de haber
		importado el módulo, pero solo tendrá efecto una vez luego de
		detectar algún joystick (es decir si se conectan o desconectan
		joysticks más tarde la función no tendrá efecto)"""
		if pygame.joystick.get_count() == 0 and not Joystick.joystickDetected:
			pygame.joystick.quit()
			pygame.joystick.init()
			Joystick.joystickDetected = pygame.joystick.get_count() > 0


		
	
	def __init__(self, robot, joystickNumber=0):
		"""Crea un joystick asociado con un objeto Robot, si
		no se idica el numero de joystick se asume 0"""
		Joystick.__refresh()
		self.js = pygame.joystick.Joystick(joystickNumber)
		self.js.init()
		self.robot = robot
	def play(self):
		"""Permite controlar un robot con un joystick analógico o
		digital, cuando se usan los botones el robot
		se mueve a velocidad máxima, si se usa un control analógico
		se puede graduar la velocidad"""
		previous = None # Permite evitar procesar eventos repetidos
		# Las siguientes 3 variables sirven para evitar conflictos
		# entre los eventos que se generan para cada eje.
		forward = False
		rotate = False
		digitalRotate = False
		while True:
			event = pygame.event.wait()
			if previous and event.type == pygame.JOYAXISMOTION and event.joy == previous.joy and event.value == previous.value and event.axis == previous.axis:
				pass
			else:
				if event.type == pygame.JOYAXISMOTION:
					previous = event
				elif event.type == pygame.JOYBUTTONDOWN and event.button == 9:
					return
				else:
					continue
		
				if event.axis == 2 or event.axis == 1:
					vel = event.value * 100
					if vel > 100:
						vel = 100
					elif vel < -100:
						vel = -100
					if event.axis == 1 and not rotate:
						self.robot.forward(-vel)
						forward = vel != 0
					elif event.axis == 2 and not forward and not digitalRotate:
						self.robot.turnLeft(-vel)
						rotate = vel != 0
				elif event.axis == 0:
					digitalRotate = True
					if event.value > 0:
						self.robot.turnRight(100)
					elif event.value < 0:
						self.robot.turnLeft(100)
					else:
						digitalRotate = False
						self.robot.stop()





# http://www.stealth-x.com/programming/driver-writing-with-python.php
import sys

_vel = 50
_tiempo = .3
def pad_up():
	print "Up"
	r.forward(_vel, _tiempo)
def pad_down():
	r.backward(_vel, _tiempo)
	print "Down"
def pad_left():
	print "Left"
	r.turnLeft(_vel, _tiempo)
def pad_right():
	print "Right"
	r.turnRight(_vel, _tiempo)

jmap = {
	0x800204: pad_up,
	0x7f0204: pad_down,
	0x800203: pad_left,
	0x7f0203: pad_right,
}

def key(cmd):
	return cmd[5] * 65536 + cmd[6] * 256 + cmd[7]


joy = open("/dev/input/js1", "r")

################
from robot import *
#try:
#	b = Board("/dev/ttyUSB1")
#except:
b = Board("/dev/ttyUSB0")
b.report()
r = Robot(b, 1)
################

byte = joy.read(1)
count = 0
cmd = []
def salir():
	b.exit()
	exit()
jmap[0x000109] = salir
while byte:
	if count == 8:
		try:
			jmap[key(cmd)]()
		except KeyError:
			pass
		#sys.stdout.write("0x")
		#for b in cmd:
		#	sys.stdout.write("{0:02x} ".format(b))
		#sys.stdout.write("\n")
		count = 0
		cmd = []
	
	cmd.append(ord(byte))
	count += 1
	byte = joy.read(1)

joy.close()

b.exit()



#!/bin/python
# -*- coding: utf-8 -*-
from robot import Board, Robot
from time import *


b = Board('/dev/ttyUSB0')
r = Robot(b,1)

ctime = time()
#avanza hasta que detecta una línea
r.forward(30)

while time() - ctime < 30:
	izq, der = r.getLine()
	if izq > 500:
		while izq > 500 and time() - ctime < 30:
			r.turnRight(30, 0.2)
	if der > 500:
		while izq > 500 and time() - ctime < 30:
			r.turnLeft(30, 0.2)
	r.forward(30)
r.stop()
b.exit()



#!/usr/bin/python
#-*- encoding: utf-8 -*-
###############################################################################
# Copyright (C) 2012 Fernando López <flopez AT linti.unlp.edu.ar>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program. 
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################

class SensesModel():
    def __init__(self):
        self.wheelLeft = tk.StringVar()
        self.wheelRight = tk.StringVar()
        self.ping = tk.StringVar()
        self.battery = tk.StringVar()

class SensesGUI():
    def __init__(self, root, model):
        self.root = root
        root.title("Senses")
        self.frame = tk.Frame(root)
        self.frame.pack(expand = 1, padx = 5, pady = 5)
        #self.frame["padding"] = 5
        label = tk.Label(self.frame)
        label["text"] = "Ping:"
        label.grid(row = 0, sticky = tk.W)
        self.ping = tk.Entry(self.frame)
        self.ping.grid(row = 0, column = 1)
        label = tk.Label(self.frame)
        label["text"] = "Sensores de las ruedas:"
        label.grid(row = 1, columnspan = 2)
        self.wheelLeft = tk.Entry(self.frame)
        self.wheelLeft.grid(row = 2, column = 0)
        self.wheelRight = tk.Entry(self.frame)
        self.wheelRight.grid(row = 2, column = 1)
        label = tk.Label(self.frame)
        label["text"] = "Medidor de batería: "
        label.grid(row = 3, sticky = tk.W)
        self.battery = tk.Entry(self.frame)
        self.battery.grid(row = 3, column = 1)
        self.model = model
        self._setEntries()
        
    def _setEntries(self):
        self.wheelLeft["textvariable"] = self.model.wheelLeft
        self.wheelRight["textvariable"] = self.model.wheelRight
        self.ping["textvariable"] = self.model.ping
        self.battery["textvariable"] = self.model.battery

def _updateModel(model, messages, root):
    values = messages.get()
    if not values:
        return
    wleft, wright = values["wheels"]
    model.ping.set(str(values["ping"]))
    model.wheelLeft.set(str(wleft))
    model.wheelRight.set(str(wright))
    model.battery.set(str(values["battery"]))
    root.after(1000, _updateModel, model, messages, root)

def _sensesDialog(messages):
    global tk
    import Tkinter # Tkinter must be imported in the process that uses it
    tk = Tkinter
    root = tk.Tk()
    model = SensesModel()
    gui = SensesGUI(root, model)
    root.after_idle(_updateModel, model, messages, root)
    root.mainloop()
    root.quit()

def _sendSensorsValues(robot):
    messages = multiprocessing.Queue()
    senses = multiprocessing.Process(target = _sensesDialog, args = (messages,))
    senses.start()
    while senses.is_alive():
        values = {
            "wheels": robot.getWheels(),
            "ping": robot.ping(),
            "battery": robot.battery()
        }
        #values = {
        #    "line": (1,1),
        #    "ping": int(time.time() % 100),
        #    "battery": int(time.time() % 100) 
        #}
        messages.put(values)
        time.sleep(1)
    senses.join()

import platform
mayor, minor, revision = platform.python_version_tuple()
if mayor != '2' or minor < '6':
    print("El módulo senses precisa una versión de Python mayor o igual a 2.6 y menor a 3")
    def senses(robot):
        print("Función no disponible en esta versión de Python")
else:
    import threading
    import multiprocessing
    import time
    def senses(robot):
        update = threading.Thread(target = _sendSensorsValues, args = (robot,))
        update.start()    

if __name__ == "__main__":
    from duinobot import *
    b = Board("/dev/ttyUSB0")
    r = Robot(b, 1)
    #r = None
    senses(r)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from distutils.core import setup
import os.path
import glob

setup(name='duinobot',
      version='0.10',
      description='API en Python para el robot Multiplo N6',
      author='Joaquin Bogado, Fernando Lopez',
      author_email='soportelihuen@linti.unlp.edu.ar',
      url='http://lihuen.linti.unlp.edu.ar',
      packages=['duinobot', 'duinobot.pyfirmata'],
      package_dir={'duinobot': '.'},
      data_files=[('/usr/share/doc/duinobot/ejemplos', glob.glob(os.path.join('ejemplos','*')))]
     )



from pyfirmata import DuinoBot, util
from N6 import N6

import sys

placa=N6();

while 1:
    for i in range(6):
        print placa.analog(i),
        print '\t',
    print '\n',
    if placa.ping() < 50:
        break


placa.exit()
